/* main.c
 *
 * simple demo that shows digital channel in out usage
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>

#include "libad.h"

#ifdef _WIN32
#include <malloc.h>
#endif


#define countof(x)   (sizeof(x)/sizeof((x)[0]))


void
usage ()
{
  printf ("usage: ex4 <driver> [-o <cha> <val>] [-i <cha>]\n"
          "  <driver>     string to pass to ad_open(), e.g. usb-pio\n"
          "               - will prompt for name\n"
          "  <cha>        number of digital port channel\n"
          "  <val>        digital out value\n");
  exit (1);
}

/* simple test
 */

int
run_test (const char *driver, int argc, char *argv[])
{
  int adh, i, chac, rc;
  uint32_t data;
  
  adh = ad_open (driver);
  if (adh == -1)
    {
      if (ad_invalid_driver_version (errno))
        printf ("failed to open %s: invalid driver version\n", driver);
      else
        printf ("failed to open %s: err = %d\n", driver, errno);

      return 1;
    }
  printf ("opened %s: handle adh = %d\n", driver, adh);
  
  rc = 0;
  for (i = 2; i < argc; i++)
    {
      if (strcmp (argv[i], "-i") == 0)
        {
          if (argv[i+1] == NULL)
            break;

          chac = atoi (argv[i+1]);
          i += 1;
          ad_set_line_direction (adh, AD_CHA_TYPE_DIGITAL_IO|chac, 0xFF); // alle Input
          rc = ad_discrete_in (adh, AD_CHA_TYPE_DIGITAL_IO|chac, 0, &data);
          if (rc == 0)
            {
              printf ("Read Digital In %d = %d (0x%02x)\n", chac, data, data);
            }
          else
            {
              printf ("error reading Digital In %d (rc=%d)\n", chac, rc);              
            }
        }

      if (strcmp (argv[i], "-o") == 0)
        {
          if (argv[i+1] == NULL
              || argv[i+2] == NULL)
            break;

          chac = atoi (argv[i+1]);
          data = atoi (argv[i+2]);
          
          i += 2;
          
          ad_set_line_direction (adh, AD_CHA_TYPE_DIGITAL_IO|chac, 0x00); // alle Output
          rc = ad_discrete_out (adh, AD_CHA_TYPE_DIGITAL_IO|chac, 0, data);
          if (rc == 0)
            printf ("Write Digital Out %d = %d (0x%02x)\n", chac, data, data);
          else
            printf ("error writing Digital Out %d (rc=%d)\n", chac, rc);              
                     
        }

    }


  ad_close (adh);  

  return rc;
}



/* main entry
 */

int
main (int argc, char *argv[])
{
  int rc;
  char *driver, *p, tmp[80];

  if (argc <= 1)
    usage ();

  if (argv[1][0] == '-')
    {
      printf ("driver to open: ");
      fgets (tmp, sizeof(tmp), stdin);

      p = strchr (tmp, '\n');
      if (p)
        *p = '\0';

      driver = tmp;
    }
  else
    driver = argv[1];

  rc = run_test (driver, argc, argv);

  if (argv[1][0] == '-')
    {
      printf ("press return to continue...\n");
      fgets (tmp, sizeof(tmp), stdin);
    }

  return rc;
}
